## Inverse Constraint Learning
This repository contains the code for ICL paper. After you run any command, the
results will be logged to Wandb (online) as well as tensorboard (locally).

## Before running anything
Please install `tools` package by running `pip install .` in the root directory. Also install `torch==1.11.0,gym==0.25`.

## Wandb setup
* Create an account on Wandb (https://wandb.ai)
* Install Wandb package: `pip3 install wandb`
* Login to wandb: `wandb login`

## High level workflow

* If you face any Pyglet/OpenGL error, install `Xvfb` and prefix the command with `xvfb-run -a`.
* For the rest of the commands, replace:
    * `SEED=1/2/3/4/5/anything`
    * `BETA=anything` (if `BETA=-1` then the default, defined in the config file, is used)
    * `ENV` is defined depending on the environments:
        * Gridworld (A): `ENV=gridworldA`
        * Gridworld (B): `ENV=gridworldB`
        * CartPole (MR): `ENV=cartpoleMR`
        * CartPole (Mid): `ENV=cartpoleM`
        * HighD: `ENV=highdgap`
* Expert data (either generate OR use saved data):
    * Use saved data: `cp expert-data/data-ENV.pt data.pt`
    * Generate for HighD environment: `python3 -B expert-highD.py`
    * Generate for other environments: `python3 -B expert.py -c configs/ENV.json`
* Run methods
    * ICL, no mix, no reweighting: `python3 -B 01-icl-basic.py -c configs/ENV.json -seed SEED -beta BETA`
    * ICL, no mix: `python3 -B 02-icl-mix.py -c configs/ENV.json -seed SEED -beta BETA`
    * ICL: `python3 -B 03-icl-mix-improved.py -c configs/ENV.json -seed SEED -beta BETA`
    * GAIL-Constraint: `python3 -B 11-gail.py -c configs/gail-ENV.yaml -seed SEED`
    * ICRL: `python3 -B 12-icrl.py -c configs/icrl-ENV.yaml -seed SEED`

## Credits

Please check the individual repositories for licenses.

* ICRL code: 
  * https://github.com/shehryar-malik/icrl
  * https://github.com/Guiliang/constraint-learning
* HighD dataset
  * https://www.highd-dataset.com
  * Free for non-commercial use, but to get the dataset, you must request it. 
  * We include one sample set of assets (#17) from the dataset in the code, since it is necessary to run the HighD environment.
* Wise-Move environment
  * https://git.uwaterloo.ca/wise-lab/wise-move
  * https://github.com/ashishgaurav13/wm2
* Gridworld environment
  * https://github.com/yrlu/irl-imitation
* Gym environment and wrappers
  * https://github.com/ikostrikov/pytorch-ddpg-naf
  * https://github.com/vwxyzjn/cleanrl
  * https://github.com/openai/gym
* Normalizing flows
  * https://github.com/ikostrikov/pytorch-flows
  * https://github.com/tonyduan/normalizing-flows
  * https://github.com/VincentStimper/normalizing-flows
